"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.NETHERMIND_NAME = void 0;

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _embarkI18n = require("embark-i18n");

var _blockchain = require("./blockchain");

var _blockchainProcessLauncher = require("./blockchainProcessLauncher");

const {
  normalizeInput,
  testRpcWithEndpoint,
  testWsEndpoint
} = require('embark-utils');

const NETHERMIND_NAME = 'nethermind';
exports.NETHERMIND_NAME = NETHERMIND_NAME;

class Nethermind {
  constructor(embark) {
    this.embark = embark;
    this.embarkConfig = embark.config.embarkConfig;
    this.blockchainConfig = embark.config.blockchainConfig;
    this.locale = embark.config.locale;
    this.logger = embark.logger;
    this.client = embark.config.blockchainConfig.client;
    this.isDev = embark.config.blockchainConfig.isDev;
    this.events = embark.events;

    if (!this.shouldInit()) {
      return;
    }

    this.events.request("blockchain:node:register", NETHERMIND_NAME, {
      isStartedFn: isStartedCb => {
        this._doCheck(state => {
          console.log('Started?', (0, _stringify.default)(state));
          return isStartedCb(null, state.status === "on");
        });
      },
      launchFn: readyCb => {
        this.events.request('processes:register', 'blockchain', {
          launchFn: cb => {
            this.startBlockchainNode(cb);
          },
          stopFn: cb => {
            this.stopBlockchainNode(cb);
          }
        });
        this.events.request("processes:launch", "blockchain", err => {
          if (err) {
            this.logger.error(`Error launching blockchain process: ${err.message || err}`);
          }

          readyCb();
        });
        this.registerServiceCheck();
      },
      stopFn: async cb => {
        await this.events.request("processes:stop", "blockchain");
        cb();
      }
    });
  }

  shouldInit() {
    return this.blockchainConfig.client === NETHERMIND_NAME && this.blockchainConfig.enabled;
  }

  _getNodeState(err, version, cb) {
    if (err) return cb({
      name: "Ethereum node not found",
      status: 'off'
    });
    return cb({
      name: `${NETHERMIND_NAME} (Ethereum)`,
      status: 'on'
    });
  }

  _doCheck(cb) {
    var _context;

    if ((0, _startsWith.default)(_context = this.blockchainConfig.endpoint).call(_context, 'ws')) {
      return testWsEndpoint(this.blockchainConfig.endpoint, (err, version) => this._getNodeState(err, version, cb));
    }

    testRpcWithEndpoint(this.blockchainConfig.endpoint, (err, version) => this._getNodeState(err, version, cb));
  }

  registerServiceCheck() {
    var _context2;

    this.events.request("services:register", 'Ethereum', (0, _bind.default)(_context2 = this._doCheck).call(_context2, this), 5000, 'off');
  }

  startBlockchainNode(callback) {
    if (this.blockchainConfig.isStandalone) {
      return new _blockchain.BlockchainClient(this.blockchainConfig, {
        clientName: NETHERMIND_NAME,
        env: this.embark.config.env,
        certOptions: this.embark.config.webServerConfig.certOptions,
        logger: this.logger,
        events: this.events,
        isStandalone: true,
        fs: this.embark.fs
      }).run();
    }

    this.blockchainProcess = new _blockchainProcessLauncher.BlockchainProcessLauncher({
      events: this.events,
      env: this.embark.config.env,
      logger: this.logger,
      normalizeInput,
      blockchainConfig: this.blockchainConfig,
      locale: this.locale,
      client: this.client,
      isDev: this.isDev,
      embark: this.embark
    });
    this.blockchainProcess.startBlockchainNode(callback);
  }

  stopBlockchainNode(cb) {
    const message = (0, _embarkI18n.__)(`The blockchain process has been stopped. It can be restarted by running ${"service blockchain on".bold} in the Embark console.`);

    if (!this.blockchainProcess) {
      return cb();
    }

    this.blockchainProcess.stopBlockchainNode(() => {
      this.logger.info(message);
      cb();
    });
  }

}

module.exports = Nethermind;
//# sourceMappingURL=index.js.map